@@ -16,14 +16,15 @@ from formant.protos.model.v1 import (
     health_pb2,
     text_pb2,
     intervention_pb2,
+    file_pb2,
 )
 from typing_extensions import Literal
 
 from .exceptions import InvalidArgument, handle_agent_exceptions
 from .cancellable_stream_thread import CancellableStreamThread
 
-DEFUALT_IMAGE_CONTENT_TYPE = "image/jpg"
-DEFAULT_SEVERITY_TYPE = "info"
+DEFUALT_IMAGE_CONTENT_TYPE = "image/jpg"  # type: Literal["image/jpg"]
+DEFAULT_SEVERITY_TYPE = "info"  # type: Literal["info"]
 ALLOWED_IMAGE_CONTENT_TYPES = ["image/jpg", "image/jpeg", "image/png"]
 ALLOWED_SEVERITY_TYPES = ["info", "warning", "critical", "error"]
 
@@ -45,9 +46,9 @@ def set_severity_pb(
     return severity_pb
 
 
-def validate_literal_type(
+def validate_string_in_array(
     given,  # type: Any
-    expected_types,  # type: List[type]
+    expected_types,  # type: List[str]
     input_name,  # type: str
 ):
     method_name = sys._getframe(1).f_code.co_name
@@ -307,9 +308,7 @@ class Client:
             numeric_set.numerics.extend(
                 [
                     math_pb2.NumericSetEntry(
-                        value=v[0],
-                        label=k,
-                        unit=v[1] if v[1] else None,
+                        value=v[0], label=k, unit=v[1] if v[1] else None,
                     )
                 ]
             )
@@ -328,7 +327,8 @@ class Client:
         stream,  # type: str
         value=None,  # type: Optional[bytes]
         url=None,  # type: Optional[str]
-        content_type="image/jpg",  # type: Literal["image/jpg", "image/png", "video/h264"] # noqa: E501
+        content_type="image/jpg",
+        # type: Literal["image/jpg", "image/png", "video/h264"]
         tags=None,  # type: Optional[Dict[str, str]]
         timestamp=None,  # type: Optional[int]
     ):
@@ -501,6 +501,51 @@ class Client:
             )
         )
 
+    @handle_agent_exceptions
+    def post_file(
+        self,
+        stream,  # type: str
+        url=None,  # type: str
+        filename=None,  # type: Optional[str]
+        tags=None,  # type: Optional[Dict[str, str]]
+        timestamp=None,  # type: Optional[int]
+    ):
+        """
+        Post a file to a telemetry stream.
+
+        :param stream: The name of the Formant stream to post the file on
+        :param url: The file url: path to local file or valid remote URL for
+            remote files
+        :param filename: The file name: name displayed inside Formant module
+        :param tags: Tags to include on the posted file
+        :param timestamp: Unix timestamp in milliseconds for the posted file.
+            Uses the current time by default
+
+        .. highlight:: python
+        .. code-block:: python
+
+            from formant.sdk.agent.v1 import Client
+
+            fclient = Client()
+            fclient.post_file(
+                "example.file",
+                /home/user/Desktop/data/planets.csv,
+                planets.csv,
+            )
+        """
+
+        validate_type(filename, [str])
+        validate_type(url, [str])
+
+        self.agent_stub.PostData(
+            datapoint_pb2.Datapoint(
+                stream=stream,
+                file=file_pb2.File(url=url, filename=filename),
+                tags=tags if tags else {},
+                timestamp=timestamp if timestamp else current_timestamp(),
+            )
+        )
+
     def register_telemetry_listener_callback(
         self,
         f,  # type: Callable[[datapoint_pb2.Datapoint], None]
@@ -629,7 +674,7 @@ class Client:
 
         validate_type(message, [str])
         validate_type(severity, [str])
-        validate_literal_type(severity, ALLOWED_SEVERITY_TYPES, "severity")
+        validate_string_in_array(severity, ALLOWED_SEVERITY_TYPES, "severity")
 
         severity_pb = set_severity_pb(severity)
 
@@ -689,9 +734,7 @@ class Client:
         :param datapoint: A datapoint related to the command
         """
         response = commands_pb2.CommandResponse(
-            request_id=request_id,
-            success=success,
-            datapoint=datapoint,
+            request_id=request_id, success=success, datapoint=datapoint,
         )
         request = agent_pb2.SendCommandResponseRequest(response=response)
         self.agent_stub.SendCommandResponse(request)
@@ -725,8 +768,7 @@ class Client:
         )
 
     def unregister_command_request_callback(
-        self,
-        f,  # type: Callable[[commands_pb2.CommandRequest], None]
+        self, f,  # type: Callable[[commands_pb2.CommandRequest], None]
     ):
         # type: (...) -> None
         """
@@ -792,8 +834,7 @@ class Client:
         return self.agent_stub.GetTeleopInfo(request)
 
     def register_teleop_heartbeat_callback(
-        self,
-        f,  # type: Callable[[agent_pb2.GetTeleopHeartbeatStreamResponse], None]
+        self, f,  # type: Callable[[agent_pb2.GetTeleopHeartbeatStreamResponse], None]
     ):
         # type : (...) -> None
         """
@@ -862,8 +903,7 @@ class Client:
         )
 
     def unregister_custom_data_channel_message_callback(
-        self,
-        f,  # type: Callable[[], None]
+        self, f,  # type: Callable[[], None]
     ):
         # type : (...) -> None
         """
@@ -1127,7 +1167,8 @@ class Client:
         options,  # type: List[str]
         hint,  # type: int
         url=None,  # type: str
-        content_type=DEFUALT_IMAGE_CONTENT_TYPE,  # type: Literal["image/jpg", "image/png"] # noqa: E501
+        content_type=DEFUALT_IMAGE_CONTENT_TYPE,
+        # type: Literal["image/jpg", "image/png"]
         timestamp=None,  # type: Optional[int]
         severity=DEFAULT_SEVERITY_TYPE,
         # type: Literal["info", "warning", "critical", "error"]
@@ -1171,8 +1212,10 @@ class Client:
         validate_type(hint, [int])
         validate_type(severity, [str])
         validate_type(content_type, [str])
-        validate_literal_type(content_type, ALLOWED_IMAGE_CONTENT_TYPES, "content_type")
-        validate_literal_type(severity, ALLOWED_SEVERITY_TYPES, "severity")
+        validate_string_in_array(
+            content_type, ALLOWED_IMAGE_CONTENT_TYPES, "content_type"
+        )
+        validate_string_in_array(severity, ALLOWED_SEVERITY_TYPES, "severity")
         for option in options:
             validate_type(option, [str])
 
@@ -1219,7 +1262,7 @@ class Client:
         """
         validate_type(instruction, [str])
         validate_type(severity, [str])
-        validate_literal_type(severity, ALLOWED_SEVERITY_TYPES, "severity")
+        validate_string_in_array(severity, ALLOWED_SEVERITY_TYPES, "severity")
 
         severity_pb = set_severity_pb(severity)
 
@@ -1237,7 +1280,8 @@ class Client:
         labels,  # type: Dict[str, str]
         hint,  # type:List[intervention_pb2.LabeledPolygon]
         url=None,  # type: str
-        content_type=DEFUALT_IMAGE_CONTENT_TYPE,  # type: Literal["image/jpg", "image/png"] # noqa: E501
+        content_type=DEFUALT_IMAGE_CONTENT_TYPE,
+        # type: Literal["image/jpg", "image/png"]
         timestamp=None,  # type: Optional[int]
         severity=DEFAULT_SEVERITY_TYPE,
         # type: Literal["info", "warning", "critical", "error"]
@@ -1281,8 +1325,10 @@ class Client:
         validate_type(url, [str])
         validate_type(content_type, [str])
         validate_type(severity, [str])
-        validate_literal_type(severity, ALLOWED_SEVERITY_TYPES, "severity")
-        validate_literal_type(content_type, ALLOWED_IMAGE_CONTENT_TYPES, "content_type")
+        validate_string_in_array(severity, ALLOWED_SEVERITY_TYPES, "severity")
+        validate_string_in_array(
+            content_type, ALLOWED_IMAGE_CONTENT_TYPES, "content_type"
+        )
         validate_type(labels, [List])
         validate_type(labels, [dict])
         for label in hint:
